#
# Copyright 2019 Andrey Galkin <andrey@futoin.org>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

from ..runtimetool import RuntimeTool
from ..buildtool import BuildTool


class phoenixTool(RuntimeTool, BuildTool):
    """
    Phoenix - a productive web framework that does not compromise speed or maintainability.

Home: https://phoenixframework.org/

Notes on tuning:
* .tune.prepare
* .tune.build

Note: MIX_ENV is set based on mixEnv or .env.type

It is expected that the server is installed as dependency in the project.
"""

    __slots__ = ()

    def getDeps(self):
        return ['mix']

    def _installTool(self, env):
        pass

    def initEnv(self, env):
        self._have_tool = True

        try:
            self._executil.callExternal(
                [env['mixBin'], 'help', 'phx'], verbose=False)
        except:
            self._info(
                'Tool "{0}" must be installed as project dependency'.format(self._name))

    def onPrepare(self, config):
        target = self._getTune(
            config, 'prepare', ['phx.digest.clean'])
        cmd = [config['env']['mixBin']] + target
        self._executil.callMeaningful(cmd)

    def onBuild(self, config):
        target = self._getTune(
            config, 'build', ['phx.digest'])
        cmd = [config['env']['mixBin']] + target
        self._executil.callMeaningful(cmd)

    def tuneDefaults(self, env):
        return {
            'internal': True,
            'minMemory': '128M',
            'debugOverhead': '32M',
            'connMemory': '32K',
            'debugConnOverhead': '64K',
            'socketTypes': ['tcp', 'tcp6'],
            'socketType': 'tcp',
            'scalable': False,
            'reloadable': False,
            'multiCore': True,
            'maxRequestSize': '1M',
            'socketProtocol': 'http',
        }

    def onRun(self, config, svc, args):
        svc_tune = svc['tune']

        # ---
        mix_env = {}

        try:
            mix_env['PORT'] = str(svc_tune['socketPort'])
            mix_env['HOST'] = svc_tune['socketAddress']
        except KeyError:
            pass

        self._environ.update(mix_env)

        # ---
        mix_args = [
            '--no-compile',
        ]

        # ---

        cmd = [
            config['env']['mixBin'],
            'phx.server',
        ] + mix_args + args

        self._executil.callInteractive(cmd)



#
# Copyright 2015-2019 Andrey Galkin <andrey@futoin.org>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

from ..util.log import *

__all__ = ('LogMixIn')


class LogMixIn(object):
    __slots__ = ()

    def _info(self, msg, label=None):
        if label:  # for backward compatibility
            infoLabel(label, msg)
        else:
            info(msg)

    def _infoLabel(self, label, msg):
        infoLabel(label, msg)

    def _warn(self, msg):
        warn(msg)

    def _errorExit(self, msg):
        errorExit(msg)



#
# Copyright 2019 Andrey Galkin <andrey@futoin.org>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

from ..runenvtool import RunEnvTool


class erlangTool(RunEnvTool):
    """
    Erlang OTP.

Home: https://www.erlang.org/
"""

    __slots__ = ()

    def _installTool(self, env):
        install = self._install

        install.dpkg(
            env, 'erlang-solutions',
            'https://packages.erlang-solutions.com/erlang-solutions_1.0_all.deb',
            True)

        install.generic(['erlang'])

    def uninstallTool(self, env):
        pass

    def initEnv(self, env):
        super(erlangTool, self).initEnv(env, 'erl')



#
# Copyright 2019 Andrey Galkin <andrey@futoin.org>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

from ..rmstool import RmsTool
from .piptoolmixin import PipToolMixIn


class awsTool(PipToolMixIn, RmsTool):
    """Generic AWS CLI and S3 as RMS.

Home: https://docs.aws.amazon.com/cli/index.html
"""
    __slots__ = ()

    def getDeps(self):
        return ['pip']

    def _pipName(self):
        return 'awscli'

    def rmsUpload(self, config, rms_pool, package_list):
        ospath = self._ospath
        bucket, path = self._getS3Repo(config)

        for package in package_list:
            package_basename = ospath.basename(package)
            self.onExec(config['env'], [
                's3', 'cp', package,
                's3://{0}{1}/{2}/{3}'.format(bucket,
                                             path, rms_pool, package_basename),
            ], False)

    def rmsPromote(self, config, src_pool, dst_pool, package_list):
        ospath = self._ospath
        bucket, path = self._getS3Repo(config)

        for package in package_list:
            package_basename = ospath.basename(package)
            self.onExec(config['env'], [
                's3', 'cp',
                's3://{0}{1}/{2}/{3}'.format(bucket,
                                             path, src_pool, package_basename),
                's3://{0}{1}/{2}/{3}'.format(bucket,
                                             path, dst_pool, package_basename),
            ], False)

    def rmsGetList(self, config, rms_pool, package_hint):
        bucket, path = self._getS3Repo(config)

        res = self._executil.callExternal([
            config['env']['awsBin'],
            's3', 'ls',
            's3://{0}{1}/{2}/'.format(bucket, path, rms_pool)])
        return [r.split()[-1] for r in res.rstrip().split('\n')]

    def rmsRetrieve(self, config, rms_pool, package_list):
        bucket, path = self._getS3Repo(config)

        for package in package_list:
            self.onExec(config['env'], [
                's3', 'cp',
                's3://{0}{1}/{2}/{3}'.format(bucket, path, rms_pool, package),
                package,
            ], False)

    def rmsGetHash(self, config, rms_pool, package, hash_type):
        # Note: Etag is not always MD5, so we skip
        raise NotImplementedError(self._name)

    def rmsPoolCreate(self, config, rms_pool):
        pass

    def rmsPoolList(self, config):
        return self.rmsGetList(config, '', '')

    def _getS3Repo(self, config):
        rms_repo = config['rmsRepo']
        parsed = self._ext.urlparse.urlparse(rms_repo)

        if parsed.scheme != 's3':
            self._errorExit(
                'AWS S3 repository scheme must be s3://bucket/path')

        bucket = parsed.netloc
        path = parsed.path

        if len(path) > 1:
            path = path.rstrip('/')
        else:
            path = ''

        return bucket, path
